/* Informacao sobre sistema de referencia para objetos: possui uma
base formada por 3 vetores e uma origem no espaco.

                          Y

                          A    Z
                         j|   /
                          |  /k
                          | /
                          |/
                          +------> X
                         O      i
 */



#include "defs.h"

#include <stdio.h>       // acesso a NULL

#include "arquivo.h"

#include "arq3d.h"

#include "solido.h"

#include "sr.h"



/* construtor - define base default na origem com vetores
ortogonais entre si e alinhados com o SRU.
**********************************************************/
CSR::CSR ()
{
   mAtrOrigem ();
}



/* construtor - define base default com vetores ortogonais
entre si e alinhados com o SRU.
Parametros definem origem.
**********************************************************/
CSR::CSR ( INT16 nXO, INT16 nYO, INT16 nZO )
{
   m_fXi = 1;
   m_fYi = m_fZi = 0;
   m_fXj = 0;
   m_fYj = 1;
   m_fZj = 0;
   m_fXk = m_fYk = 0;
   m_fZk = 1;

   m_nXO = nXO;
   m_nYO = nYO;
   m_nZO = nZO;
}



/* construtor - define base e origem por parametros.
**********************************************************/
CSR::CSR ( INT16 nXO, INT16 nYO, INT16 nZO,
           float fXi, float fYi, float fZi,
           float fXj, float fYj, float fZj,
           float fXk, float fYk, float fZk )
{
   m_fXi = fXi;
   m_fYi = fYi;
   m_fZi = fZi;
   m_fXj = fXj;
   m_fYj = fYj;
   m_fZj = fZj;
   m_fXk = fXk;
   m_fYk = fYk;
   m_fZk = fZk;

   m_nXO = nXO;
   m_nYO = nYO;
   m_nZO = nZO;
}



/* Transforma solido conforme sistema de referencia
**********************************************************/
void CSR::operator >> ( CSolido& oS )
{
   // altera vertices conforme base do sistema de coordenadas
   for ( UINT16 ucV = 0; ucV < oS.m_uNumVertices; ucV ++ ) {

      INT16
         nX = * ( oS.m_panX + ucV ),
         nY = * ( oS.m_panY + ucV ),
         nZ = * ( oS.m_panZ + ucV );
      * ( oS.m_panX + ucV ) = (INT16) ( (float) nX * m_fXi + nY * m_fYi + nZ * m_fZi );
      * ( oS.m_panY + ucV ) = (INT16) ( (float) nX * m_fXj + nY * m_fYj + nZ * m_fZj );
      * ( oS.m_panZ + ucV ) = (INT16) ( (float) nX * m_fXk + nY * m_fYk + nZ * m_fZk );
   }
}



/* Transforma e copia valores operados para outro solido
Usa ponto fixo 16.16.
**********************************************************/
void CSR::mAplicaBase ( CSolido& oS, CSolido& oSTr )
{
   INT
      nXi = (INT) ( m_fXi * 65536 ),
      nYi = (INT) ( m_fYi * 65536 ),
      nZi = (INT) ( m_fZi * 65536 ),
      nXj = (INT) ( m_fXj * 65536 ),
      nYj = (INT) ( m_fYj * 65536 ),
      nZj = (INT) ( m_fZj * 65536 ),
      nXk = (INT) ( m_fXk * 65536 ),
      nYk = (INT) ( m_fYk * 65536 ),
      nZk = (INT) ( m_fZk * 65536 );

   // altera vertices conforme base do sistema de coordenadas
   for ( UINT16 ucV = 0; ucV < oS.m_uNumVertices; ucV ++ ) {

      INT16
         nX = * ( oS.m_panX + ucV ),
         nY = * ( oS.m_panY + ucV ),
         nZ = * ( oS.m_panZ + ucV );
      * ( oSTr.m_panX + ucV ) = (INT16) ( nX * nXi >> 16 ) + ( nY * nYi >> 16 ) + ( nZ * nZi >> 16 );
      * ( oSTr.m_panY + ucV ) = (INT16) ( nX * nXj >> 16 ) + ( nY * nYj >> 16 ) + ( nZ * nZj >> 16 );
      * ( oSTr.m_panZ + ucV ) = (INT16) ( nX * nXk >> 16 ) + ( nY * nYk >> 16 ) + ( nZ * nZk >> 16 );
   }

   oSTr.m_uNumVertices = oS.m_uNumVertices;
}



/* Transforma componentes conforme sistema de referencia
**********************************************************/
void CSR::mAplicaBase ( float& fX, float& fY, float& fZ )
{
   float
      fXa = fX * m_fXi + fY * m_fYi + fZ * m_fZi,
      fYa = fX * m_fXj + fY * m_fYj + fZ * m_fZj,
      fZa = fX * m_fXk + fY * m_fYk + fZ * m_fZk;
   fX = fXa;
   fY = fYa;
   fZ = fZa;
}



/* Transforma, translada e copia valores operados para outro solido
Usa ponto fixo 16.16.
**********************************************************/
void CSR::mAplica ( CSolido& oS, CSolido& oSTr )
{
   INT
      nXi = (INT) ( m_fXi * 65536 ),
      nYi = (INT) ( m_fYi * 65536 ),
      nZi = (INT) ( m_fZi * 65536 ),
      nXj = (INT) ( m_fXj * 65536 ),
      nYj = (INT) ( m_fYj * 65536 ),
      nZj = (INT) ( m_fZj * 65536 ),
      nXk = (INT) ( m_fXk * 65536 ),
      nYk = (INT) ( m_fYk * 65536 ),
      nZk = (INT) ( m_fZk * 65536 );

   // altera vertices conforme base do sistema de coordenadas
   for ( UINT16 ucV = 0; ucV < oS.m_uNumVertices; ucV ++ ) {

      INT16
         nX = * ( oS.m_panX + ucV ),
         nY = * ( oS.m_panY + ucV ),
         nZ = * ( oS.m_panZ + ucV );
      * ( oSTr.m_panX + ucV ) = (INT16) ( nX * nXi >> 16 ) + ( nY * nYi >> 16 ) + ( nZ * nZi >> 16 ) + m_nXO;
      * ( oSTr.m_panY + ucV ) = (INT16) ( nX * nXj >> 16 ) + ( nY * nYj >> 16 ) + ( nZ * nZj >> 16 ) + m_nYO;
      * ( oSTr.m_panZ + ucV ) = (INT16) ( nX * nXk >> 16 ) + ( nY * nYk >> 16 ) + ( nZ * nZk >> 16 ) + m_nZO;
   }

   oSTr.m_uNumVertices = oS.m_uNumVertices;
}



/* Transforma e translada componentes conforme sistema de referencia
**********************************************************/
void CSR::mAplica ( float& fX, float& fY, float& fZ )
{
   float
      fXa = fX * m_fXi + fY * m_fYi + fZ * m_fZi,
      fYa = fX * m_fXj + fY * m_fYj + fZ * m_fZj,
      fZa = fX * m_fXk + fY * m_fYk + fZ * m_fZk;
   fX = fXa + m_nXO;
   fY = fYa + m_nYO;
   fZ = fZa + m_nZO;
}



/* Altera componentes da base de vetores para os valores originais
**********************************************************/
void CSR::mAtrOrigemBase ()
{
   m_fXi = 1;
   m_fYi = 0;
   m_fZi = 0;
   m_fXj = 0;
   m_fYj = 1;
   m_fZj = 0;
   m_fXk = 0;
   m_fYk = 0;
   m_fZk = 1;
}



/* Altera componentes da base de vetores e origem do SR
 para os valores originais
**********************************************************/
void CSR::mAtrOrigem ()
{
   m_fXi = 1;
   m_fYi = 0;
   m_fZi = 0;
   m_fXj = 0;
   m_fYj = 1;
   m_fZj = 0;
   m_fXk = 0;
   m_fYk = 0;
   m_fZk = 1;

   m_nXO = m_nYO = m_nZO = 0;
}
